#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import json
import collections
import copy


class Link(object):
    '''
    Class which defines a HAL Link object
    '''

    href = None
    title = None

    def __init__(self, href, title=None):
        '''
        @summary: constructor
        @param href: href target
        @param title: title of the link
        @result: HAL Link object
        '''
        self.href = href
        self.title = title

    def to_dict(self):
        '''
        @summary: returns the HAL Link object as a python dict
        @result: python dict
        '''
        if self.title:
            return {"href": self.href, "title": self.title}
        else:
            return {"href": self.href}


class Resource(object):
    '''
    Class which defines a HAL Resource object
    '''

    properties = None
    links = None
    embedded_resources = None

    def __init__(self, href, properties=None):
        '''
        @summary: constructor
        @param href: href target
        @param properties: python dict of properties as key => value
        @result: HAL Resource object
        '''
        if properties:
            self.properties = copy.copy(properties)
        else:
            self.properties = {}
        self.links = {}
        self.embedded_resources = {}
        self.add_link("self", Link(href))

    def add_property(self, name, value):
        '''
        @summary: adds a property to the resource
        @param name: name
        @param value: value
        '''
        self.properties[name] = value

    def add_link(self, rel, link, multiple=False):
        '''
        @summary: adds a link to the resource
        @param rel: rel type
        @param link: HAL Link object to add
        @param multiple: if True, allow multiple links for the rel type
        '''
        if rel not in self.links and multiple:
            self.links[rel] = []
        if multiple:
            self.links[rel].append(link)
        else:
            self.links[rel] = link

    def add_embedded_resource(self, collection_name, resource):
        '''
        @summary: adds an embedded resource to the resource
        @param collection_name: name of the collection
        @param resource: HAL Resource object to add as embedded resource
        '''
        if collection_name not in self.embedded_resources:
            self.embedded_resources[collection_name] = []
        self.embedded_resources[collection_name].append(resource)

    def to_dict(self):
        '''
        @summary: returns the HAL Resource object as a python dict
        @result: python dict

        Note: the serialization is recursive (on embedded resources and on links)
        '''
        tmp = {}
        for name in self.properties:
            tmp[name] = self.properties[name]
        tmp["_links"] = {}
        for rel in self.links:
            if isinstance(self.links[rel], collections.Iterable):
                tmp["_links"][rel] = [x.to_dict() for x in self.links[rel]]
            else:
                tmp["_links"][rel] = self.links[rel].to_dict()
        if len(self.embedded_resources) > 0:
            tmp["_embedded"] = {}
        for collection_name in self.embedded_resources:
            li = [x.to_dict() for x in (self.embedded_resources[collection_name])]
            tmp["_embedded"][collection_name] = li
        return tmp

    def to_json(self):
        '''
        @summary: returns the HAL Resource object as JSON
        @result: JSON serialization of the HAL Resource object
        '''
        return json.dumps(self.to_dict(), indent=4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from tornado.options import define
from tornado.options import options as tornado_options

define("port", default=8888, type=int, metavar="PORT",
       help="main port (of the lock manager)", group="rdlm")
define("admin_userpass_file", default="yes", type=str, metavar="ADMIN_USERPASS_FILE",
       help="the full path of an admin userpass file (special values : no => no admin requests, \
             yes => no auth for admin requests)", group="rdlm")


class Options(object):
    '''Class to store command line options'''

    @classmethod
    def port(cls):
        '''
        @summary: returns the main port of the daemon
        @result: the port of the daemon (as an integer)
        '''
        return tornado_options.port

    @classmethod
    def admin_userpass_file(cls):
        '''
        @summary: returns the admin userpass file
        @result: the full path of the admin userpass file
        '''
        return tornado_options.admin_userpass_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler
from rdlm.lock import Lock
import tornado.web
import tornado.gen
import tornado.ioloop
from rdlm.lock import LOCK_MANAGER_INSTANCE
import functools


class LocksHandler(RequestHandler):
    """Class which handles the /locks/[resource] URL"""

    def on_active_wrapper(self, name, lock):
        '''
        @summary: wrapper method to invoke on_active method through tornado ioloop
        @param name: name of the resource
        @param lock: lock object
        '''
        tornado.ioloop.IOLoop.instance().add_callback(functools.partial(self.on_active,
                                                                        name, lock))

    def on_delete_wrapper(self, timeout=True):
        '''
        @summary: wrapper method to invoke on_delete method through tornado ioloop
        @param timeout: if True, the delete is made by a timeout
        '''
        f = functools.partial(self.on_delete, timeout=timeout)
        tornado.ioloop.IOLoop.instance().add_callback(f)

    def on_active(self, name, lock):
        '''
        @summary: method called when the lock becomes active
        @param name: name of the resource
        @param lock: lock object

        The method returns an HTTP/201 in this case with
        the corresponding Location header
        '''
        url = "%s%s" % (self.get_base_url(self.request), self.reverse_url("lock", name, lock.uid))
        self.send_status(201, message="lock acquired at %s" % url, headers={"Location": url})

    def on_delete(self, timeout=True):
        '''
        @summary: method called when the wait for the lock is over
        @param timeout: if True, the delete is made by a timeout

        The method returns an HTTP/408 or an HTTP/409 in this case
        (depending if the delete is made by a timeout or an admin request)
        '''
        if not(timeout):
            self.send_error(status_code=409, message="lock request deleted")
        else:
            self.send_error(status_code=408, message="lock request (wait) timeout")

    @tornado.web.asynchronous
    def post(self, name):
        '''
        @summary: deals with POST request (acquiring locks on resource)
        @param name: name of the resource
        '''
        raw_body = self.request.body.decode('utf-8')
        if len(raw_body) == 0:
            self.send_error(status_code=400, message="empty body")
            return
        lock = Lock.from_json(name, raw_body)
        if not(lock):
            self.send_error(status_code=400, message="invalid json body")
            return
        lock.set_callbacks(functools.partial(self.on_active_wrapper, name, lock),
                           self.on_delete_wrapper)
        LOCK_MANAGER_INSTANCE.add_lock(name, lock)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler, admin_authenticated
from rdlm.lock import LOCK_MANAGER_INSTANCE
from rdlm.hal import Resource


class ResourceHandler(RequestHandler):
    """Class which handles the /resources/[resource] URL"""

    SUPPORTED_METHODS = ['GET', 'DELETE']

    @admin_authenticated
    def delete(self, name):
        '''
        @summary: deals with DELETE request (deleting the given resource)
        @param name: name of the resource
        '''
        res = LOCK_MANAGER_INSTANCE.remove_resource(name)
        if res:
            self.send_status(204)
        else:
            self.send_error(404, message="no resource (with locks) found")

    @admin_authenticated
    def get(self, name):
        '''
        @summary: deals with GET request (getting a JSON HAL of the resource)
        @param name: name of the resource
        '''
        tmp = LOCK_MANAGER_INSTANCE.get_resource_as_dict(name)
        resource = Resource(self.reverse_url("resource", name), {"name": name})
        if tmp:
            for lock_dict in tmp['locks']:
                lock = Resource(self.reverse_url("lock", name, lock_dict['uid']), lock_dict)
                resource.add_embedded_resource("locks", lock)
        self.set_header("Content-Type", "application/hal+json")
        self.finish(resource.to_json())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import tornado.testing
from rdlm.main import get_app as rdlm_get_app
import os
import base64


class AuthTestCase(tornado.testing.AsyncHTTPTestCase):

    def get_app(self):
        return rdlm_get_app()

    def tearDown(self):
        self.set_config_file(["admin_userpass_file='yes'"])
        req = tornado.httpclient.HTTPRequest(self.get_url("/resources"), method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)
        super(AuthTestCase, self).tearDown()

    # compatibility with python 2.6
    def assertIn(self, str1, str2):
        self.assertTrue(str1 in str2)

    def current_dir(self):
        return os.path.dirname(__file__)

    def set_config_file(self, keyvalues):
        config_file = "%s/conf.py" % self.current_dir()
        with open(config_file, "w") as f:
            f.write("\n".join(keyvalues) + "\n")
        tornado.options.parse_config_file(config_file)

    def set_userpass_file(self, keyvalues):
        auth_file = "%s/auth.txt" % self.current_dir()
        with open(auth_file, "w") as f:
            f.write("\n".join(keyvalues) + "\n")
        self.set_config_file(["admin_userpass_file='%s'" % auth_file])

    def test_no(self):
        self.set_config_file(["admin_userpass_file='no'"])
        req = tornado.httpclient.HTTPRequest(self.get_url("/resources"), method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 403)

    def test_yes(self):
        self.set_config_file(["admin_userpass_file='yes'"])
        req = tornado.httpclient.HTTPRequest(self.get_url("/resources"), method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)

    def test_userpass(self):
        self.set_userpass_file(["foo:bar", "foo2:bar2", "foo3:bar3"])
        req = tornado.httpclient.HTTPRequest(self.get_url("/resources"), method='DELETE')
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 401)
        b64 = base64.standard_b64encode(b"foo2:bar2")
        req = tornado.httpclient.HTTPRequest(self.get_url("/resources"),
                                             method='DELETE',
                                             headers={'Authorization': b"Basic " + b64})
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 204)
        b64 = base64.standard_b64encode(b"foo:bar2")
        req = tornado.httpclient.HTTPRequest(self.get_url("/resources"),
                                             method='DELETE',
                                             headers={'Authorization': b"Basic " + b64})
        self.http_client.fetch(req, self.stop)
        response = self.wait()
        self.assertEqual(response.code, 401)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import tornado.ioloop
import tornado.web
from tornado.httpserver import HTTPServer

import functools
import logging
import signal
import time

from rdlm.options import Options
from rdlm.hello_handler import HelloHandler
from rdlm.locks_handler import LocksHandler
from rdlm.lock_handler import LockHandler
from rdlm.resource_handler import ResourceHandler
from rdlm.resources_handler import ResourcesHandler
from rdlm.lock import LOCK_MANAGER_INSTANCE


def on_every_second():
    '''
    @summary: function called by tornado/ioloop every second

    It's used to clear expired locks
    '''
    LOCK_MANAGER_INSTANCE.clean_expired_locks()


def get_app():
    '''
    @summary: returns the tornado application
    @param unit_testing: if True, we add some handler for unit testing only
    @result: the tornado application
    '''
    url_list = [
        tornado.web.URLSpec(r"/", HelloHandler, name="hello"),
        tornado.web.URLSpec(r"/resources/([a-zA-Z0-9]+)", ResourceHandler, name="resource"),
        tornado.web.URLSpec(r"/resources", ResourcesHandler, name="resources"),
        tornado.web.URLSpec(r"/locks/([a-zA-Z0-9]+)", LocksHandler, name="locks"),
        tornado.web.URLSpec(r"/locks/([a-zA-Z0-9]+)/([a-zA-Z0-9]+)", LockHandler, name="lock")
    ]
    application = tornado.web.Application(url_list)
    return application


def get_ioloop():
    '''
    @summary: returns a configured tornado ioloop
    '''
    iol = tornado.ioloop.IOLoop.instance()
    tornado.ioloop.PeriodicCallback(on_every_second, 1000, iol).start()
    return iol


def log_is_ready():
    '''
    @summary: simple callback just to log that the daemon is ready
    '''
    logging.info("RDLM daemon is ready !")


def sigterm_handler(server, loop, signum, frame):
    logging.info("SIGTERM signal catched => scheduling webserver stop...")
    loop.add_callback(functools.partial(stop_server, server, loop))


def stop_server(server, loop):
    logging.info("Stopping webserver...")
    server.stop()
    if loop:
        logging.info("Webserver stopped => scheduling main loop stop...")
        loop.add_timeout(time.time() + 5, functools.partial(stop_loop, loop))
    else:
        logging.info("Webserver stopped !")


def stop_loop(loop):
    logging.info("Stopping main loop...")
    loop.stop()
    logging.info("Main loop stopped !")


def main():
    '''
    @summary: main function (starts the daemon)
    '''
    application = get_app()
    tornado.options.parse_command_line()
    server = HTTPServer(application)
    server.listen(Options.port())
    iol = get_ioloop()
    iol.add_callback(log_is_ready)
    signal.signal(signal.SIGTERM, lambda s, f: sigterm_handler(server, iol, s, f))
    try:
        iol.start()
    except KeyboardInterrupt:
        stop_server(server, None)
    logging.info("RDLM daemon is stopped !")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import uuid
import datetime
import collections
import json
import logging


class Lock(object):
    '''
    Class which defines a Lock object

    The lock object has a status :
    - active (the lock is acquired)
    - not active (the lock is not acquired)
    '''

    __active = False
    uid = None
    title = None
    resource_name = None
    lifetime = 0
    wait = 0
    wait_since = None
    wait_expires = None
    active_since = None
    active_expires = None
    __active_callback = None
    __delete_callback = None

    def __init__(self, resource_name, title, wait, lifetime):
        '''
        @summary: lock constructor
        @param resource_name: name of the resource to lock
        @param title: title of the lock
        @param wait: wait max duration (in seconds)
        @param lifetime: timeout of the lock (in seconds)
        @result: lock object (not active)
        '''
        self.resource_name = resource_name
        self.lifetime = lifetime
        self.title = title
        self.uid = str(uuid.uuid4()).replace('-', '')
        self.wait = wait
        self.lifetime = lifetime
        self.wait_since = datetime.datetime.now()
        self.wait_expires = self.wait_since + datetime.timedelta(seconds=wait)

    def delete(self, timeout=True):
        '''
        @summary: explicit destructor
        @param timeout: if True, the delete is made by a timeout
        '''
        if self.__delete_callback:
            (self.__delete_callback)(timeout=timeout)
        self.reset_callbacks()

    @classmethod
    def from_json(cls, resource_name, json_string):
        '''
        @summary: class method which build a lock object from the corresponding json string
        @param resource_name: name of the resource
        @param json_string: json string
        @result: lock object (not active) or None if unseriliaze errors
        '''
        try:
            tmp = json.loads(json_string)
            title = tmp['title']
            wait = int(tmp['wait'])
            lifetime = int(tmp['lifetime'])
        except (KeyError, ValueError):
            return None
        return Lock(resource_name, title, wait, lifetime)

    def to_dict(self):
        '''
        @summary: method which dumps the lock object as a python dict
        @result: python dict
        '''
        tmp = {"uid": self.uid,
               "title": self.title,
               "wait": self.wait,
               "lifetime": self.lifetime,
               "active": self.__active}
        if self.__active:
            tmp['active_since'] = self.active_since.isoformat()
            tmp['active_expires'] = self.active_expires.isoformat()
        else:
            tmp['wait_since'] = self.wait_since.isoformat()
            tmp['wait_expires'] = self.wait_expires.isoformat()
        return tmp

    def to_json(self):
        '''
        @summary: method which dumps the lock object as a json string
        @result: json string
        '''
        tmp = self.to_dict()
        return json.dumps(tmp, indent=4, sort_keys=True)

    def set_active(self):
        '''
        @summary: change the status of the lock to active (corresponding callback is invoked)
        '''
        self.wait_since = None
        self.wait_expires = None
        self.__active = True
        self.active_since = datetime.datetime.now()
        self.active_expires = self.active_since + datetime.timedelta(seconds=self.lifetime)
        if self.__active_callback:
            (self.__active_callback)()
        self.reset_callbacks()

    def set_callbacks(self, active_callback, delete_callback):
        '''
        @summary: setter for callbacks
        @param active_callback: active callback
        @param delete_callback: delete callback
        '''
        self.__active_callback = active_callback
        self.__delete_callback = delete_callback

    def reset_callbacks(self):
        '''
        @summary: reset the lock callbacks
        '''
        self.__active_callback = None
        self.__delete_callback = None

    def is_expired(self):
        '''
        @summary: returns True is the lock is expired (wait timeout or lifetime timeout
                  depending on the status)
        @result: True (the lock is expired) or False
        '''
        if self.__active:
            return (datetime.datetime.now() > self.active_expires)
        else:
            return (datetime.datetime.now() > self.wait_expires)


class Resource(object):
    """Class which defines a Resource object"""

    name = None
    active_lock = None
    __waiting_locks = None

    def __init__(self, name):
        '''
        @summary: constructor
        @param name: name of the resource
        '''
        self.name = name
        self.active_lock = None
        self.__waiting_locks = collections.deque()

    def to_dict(self):
        '''
        @summary: method which dumps the resource as a python dict
        @result: python dict
        '''
        tmp = {}
        tmp["name"] = self.name
        tmp["locks"] = []
        if self.active_lock:
            tmp["locks"].append(self.active_lock.to_dict())
        for waiting_lock in self.__waiting_locks:
            tmp["locks"].append(waiting_lock.to_dict())
        return tmp

    def to_json(self):
        '''
        @summary: method which dumps the resource object as a json string
        @result: json string
        '''
        tmp = self.to_dict()
        return json.dumps(tmp, indent=4, sort_keys=True)

    def delete(self, uid=None):
        '''
        @summary: delete all locks of the resource (uid=None) or a specific lock (uid!=None)
        @param uid: uid of the lock to delete (or None to delete all)
        @result: True if there was at least a lock, False else
        '''
        tmp_queue = collections.deque()
        res = False
        while True:
            try:
                lock = self.__waiting_locks.popleft()
                if not(uid):
                    lock.delete(timeout=False)
                    res = True
                else:
                    if lock.uid == uid:
                        lock.delete(timeout=False)
                        res = True
                    else:
                        tmp_queue.append(lock)
            except IndexError:
                break
        self.__waiting_locks = tmp_queue
        res = res or self.remove_active_lock(timeout=False, uid=uid)
        return res

    def get(self, uid):
        '''
        @summary: return the lock with the given uid
        @param uid: uid of the lock to get
        @result: lock object (or None)
        '''
        for lock in self.__waiting_locks:
            if lock.uid == uid and not(lock.is_expired()):
                return lock
        if self.active_lock and not(self.active_lock.is_expired()):
            if self.active_lock.uid == uid:
                return self.active_lock

    def remove_active_lock(self, timeout=True, uid=None):
        '''
        @summary: remove the active lock of the resource (if any)
        @param timeout: if True, the delete is made by a timeout
        @param uid: uid of the lock to delete (or None to delete
                    the active lock without specific test)
        @result: True if there was an active lock, False else

        Of course, if there is some non expired waiting locks,
        the first one is promoted as the active lock of the resource
        '''
        if self.active_lock:
            if not(uid) or self.active_lock.uid == uid:
                self.active_lock.delete(timeout=timeout)
                self.active_lock = None
                while True:
                    try:
                        lock = self.__waiting_locks.popleft()
                        if lock.is_expired():
                            lock.delete(timeout=timeout)
                            continue
                        lock.set_active()
                        self.active_lock = lock
                        break
                    except IndexError:
                        break
                return True
        return False

    def add_lock(self, lock):
        '''
        @summary: add a lock to the resource
        @param lock: lock object

        If there is no active lock, the lock is promoted as the active
        lock for the resource

        If there is alreay an active lock, the lock is added to the
        waiting list
        '''
        if not(self.active_lock):
            lock.set_active()
            self.active_lock = lock
        else:
            self.__waiting_locks.append(lock)

    def clean_expired_locks(self):
        '''
        @summary: clean expired lock of the resource (active and waiting)
        '''
        tmp_queue = collections.deque()
        while True:
            try:
                lock = self.__waiting_locks.popleft()
                if lock.is_expired():
                    logging.warning("Expired waiting lock [%s] on [%s] => removing it" % (
                                    lock.title, self.name))
                    lock.delete()
                    continue
                tmp_queue.append(lock)
            except IndexError:
                break
        self.__waiting_locks = tmp_queue
        if self.active_lock and self.active_lock.is_expired():
            logging.warning("Expired active lock [%s] on [%s] => releasing it" % (
                            self.active_lock.title, self.name))
            self.remove_active_lock()


class LockManager(object):
    '''
    Class which managers resources

    Designed to be used as a singleton
    '''

    __resources_dict = None

    def __init__(self):
        '''
        @summary: constructor
        @result: ResourceManager object
        '''
        self.__resources_dict = {}

    def remove_all_resources(self):
        '''
        @summary: remove all resources
        '''
        resource_names = self.get_resources_names()
        for name in resource_names:
            self.remove_resource(name)
        self.__resources_dict = {}

    def get_resources_names(self):
        '''
        @summary: returns a python list with resource names
        @result: python list of strings
        '''
        return list(self.__resources_dict.keys())

    def get_resource_as_dict(self, resource_name):
        '''
        @summary: returns the given resource as a python dict
        @param resource_name: name of resource
        @result: python dict
        '''
        if resource_name not in self.__resources_dict:
            return None
        return self.__resources_dict[resource_name].to_dict()

    def remove_resource(self, resource_name):
        '''
        @summary: remove a resource (and its locks)
        @param resource_name: name of the resource
        @result: True if there was at least one lock, False else
        '''
        if resource_name in self.__resources_dict:
            res = self.__resources_dict[resource_name].delete()
            del(self.__resources_dict[resource_name])
            return res
        return False

    def add_lock(self, resource_name, lock):
        '''
        @summary: add a lock to the resource
        @param resource_name: name of the resource
        @param lock: lock object

        If there is no active lock, the lock is promoted as the active
        lock for the resource

        If there is alreay an active lock, the lock is added to the
        waiting list
        '''
        if resource_name not in self.__resources_dict:
            self.__resources_dict[resource_name] = Resource(resource_name)
        resource = self.__resources_dict[resource_name]
        resource.add_lock(lock)

    def delete_lock(self, resource_name, uid):
        '''
        @summary: delete a specific lock for the given resource
        @param resource_name: name of the resource
        @param uid: uid of the lock to delete
        @param lock: True if something has been deleted (False else)
        '''
        if resource_name not in self.__resources_dict:
            self.__resources_dict[resource_name] = Resource(resource_name)
        resource = self.__resources_dict[resource_name]
        return resource.delete(uid)

    def get_lock(self, resource_name, uid):
        '''
        @summary: get a specific lock for the given resource
        @param resource_name: name of the resource
        @param uid: uid of the lock to get
        @param lock: lock object (or None if not found)
        '''
        if resource_name not in self.__resources_dict:
            self.__resources_dict[resource_name] = Resource(resource_name)
        resource = self.__resources_dict[resource_name]
        return resource.get(uid)

    def clean_expired_locks(self):
        '''
        @summary: clean expired lock of all resources (active and waiting)
        '''
        names = self.get_resources_names()
        for name in names:
            self.__resources_dict[name].clean_expired_locks()


LOCK_MANAGER_INSTANCE = LockManager()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from rdlm.hal import Link, Resource


class HalTestCase(unittest.TestCase):

    def test_link(self):
        a = Link("/foo")
        b = Link("/foo", title="bar")
        tmp1 = a.to_dict()
        tmp2 = b.to_dict()
        self.assertEqual(tmp1['href'], "/foo")
        self.assertEqual(tmp2['href'], "/foo")
        self.assertTrue("title" not in tmp1)
        self.assertTrue("title" in tmp2)
        self.assertEqual(tmp2['title'], "bar")

    def test_resource1(self):
        a = Resource("/foo")
        tmp = a.to_dict()
        self.assertEqual(tmp['_links']['self']['href'], '/foo')

    def test_resource2(self):
        a = Resource("/foo", {'key1': 'value1', 'key2': 'value2'})
        tmp = a.to_dict()
        self.assertEqual(tmp['key1'], 'value1')
        self.assertEqual(tmp['key2'], 'value2')

    def test_resource3(self):
        a = Resource("/foo", {'key1': 'value1', 'key2': 'value2'})
        b = Resource("/bar", {'key3': 'value3', 'key4': 'value4'})
        a.add_embedded_resource("bars", b)
        tmp = a.to_dict()
        self.assertEqual(tmp["_embedded"]['bars'][0]['key3'], 'value3')
        self.assertEqual(tmp["_embedded"]['bars'][0]['key4'], 'value4')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

import tornado.web
import traceback
try:
    from httplib import responses
except ImportError:
    # Compatibility with Python3
    from http.client import responses
from rdlm.options import Options
import base64


def admin_authenticated(func):
    def wrapper(handler, *args, **kwargs):
        admin_userpass_file = Options.admin_userpass_file()
        if admin_userpass_file == "yes":
            return func(handler, *args, **kwargs)
        if admin_userpass_file == "no":
            handler.send_error(403)
            return False
        auth_header = handler.request.headers.get('Authorization')
        if auth_header is not None and auth_header.startswith('Basic '):
            tmp = auth_header.encode('ascii')
            auth_decoded = base64.decodestring(tmp[6:])
            basicauth_user, basicauth_pass = auth_decoded.decode('utf-8').split(':', 2)
            try:
                with open(admin_userpass_file, "r") as f:
                    lines = f.readlines()
                    for line in lines:
                        username, password = line.strip().split(":", 1)
                        if (basicauth_user == username) and (basicauth_pass == password):
                            return func(handler, *args, **kwargs)
            except:
                handler.send_error(500, message="bad admin userpass file")
                return False
        handler.set_status(401)
        handler.set_header('WWW-Authenticate', 'Basic realm=Restricted')
        handler.finish()
        return False
    return wrapper


class RequestHandler(tornado.web.RequestHandler):
    """Just an abstract class to store some utilities methods"""

    def get_base_url(self, request):
        '''
        @summary: returns the http://hostname:port part of an url
        @param request: incoming HttpRequest object (from tornado)
        @result: the http://hostname:port part of an url

        The base url is built from another incoming http request
        to get the correct hostname and port as choosen by the
        client (in order to avoid some problems with proxy
        configurations)
        '''
        return "%s://%s" % (request.protocol, request.host)

    def send_status(self, status_code, message=None, headers=None):
        self.set_status(status_code)
        if headers:
            for name in headers.keys():
                self.set_header(name, headers[name])
        if message:
            self.finish("<html><title>%(code)d: %(message)s</title>"
                        "<body>%(code)d: %(message)s</body></html>" % {
                        "code": status_code,
                        "message": message,
                        })
        else:
            self.finish()

    def write_error(self, status_code, **kwargs):
        if self.settings.get("debug") and "exc_info" in kwargs:
            # in debug mode, try to send a traceback
            self.set_header('Content-Type', 'text/plain')
            for line in traceback.format_exception(*kwargs["exc_info"]):
                self.write(line)
            self.finish()
        else:
            if 'message' in kwargs:
                message = kwargs['message']
            else:
                message = responses[status_code]
            self.finish("<html><title>%(code)d: %(message)s</title>"
                        "<body>%(code)d: %(message)s</body></html>" % {
                        "code": status_code,
                        "message": message,
                        })



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler, admin_authenticated
from rdlm.lock import LOCK_MANAGER_INSTANCE
from rdlm.hal import Resource


class ResourcesHandler(RequestHandler):
    """Class which handles the /resources URL"""

    SUPPORTED_METHODS = ['GET', 'DELETE']

    @admin_authenticated
    def delete(self):
        '''
        @summary: deals with DELETE request (deleting all resources)
        '''
        LOCK_MANAGER_INSTANCE.remove_all_resources()
        self.send_status(204)

    @admin_authenticated
    def get(self):
        '''
        @summary: deals with GET request (getting a JSON HAL of resources)
        '''
        resources = Resource(self.reverse_url("resources"))
        resources_names = LOCK_MANAGER_INSTANCE.get_resources_names()
        for resource_name in resources_names:
            tmp = LOCK_MANAGER_INSTANCE.get_resource_as_dict(resource_name)
            resource = Resource(self.reverse_url("resource", tmp['name']), {"name": tmp['name']})
            resources.add_embedded_resource("resources", resource)
        self.set_header("Content-Type", "application/hal+json")
        self.finish(resources.to_json())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of restful-distributed-lock-manager released under the MIT license.
# See the LICENSE file for more information.

from rdlm.request_handler import RequestHandler
from rdlm.lock import LOCK_MANAGER_INSTANCE
from rdlm.hal import Resource, Link


class LockHandler(RequestHandler):
    """Class which handles the /locks/[resource]/[uid] URL"""

    SUPPORTED_METHODS = ['GET', 'DELETE']

    def get(self, name, uid):
        '''
        @summary: deals with GET request
        @param name: name of the resource
        @param uid: uid of the lock
        '''
        lock = LOCK_MANAGER_INSTANCE.get_lock(name, uid)
        if lock:
            self.set_header('Content-Type', 'application/hal+json')
            hal_lock = Resource(href=self.reverse_url("lock", name, lock.uid),
                                properties=lock.to_dict())
            hal_resource_link = Link(href=self.reverse_url("resource", name))
            hal_lock.add_link(rel="resource", link=hal_resource_link, multiple=False)
            self.write(hal_lock.to_json())
        else:
            self.send_error(status_code=404, message="lock not found")
            return

    def delete(self, name, uid):
        '''
        @summary: deals with DELETE request (releasing a lock)
        @param name: name of the resource
        @param uid: uid of the lock
        '''
        res = LOCK_MANAGER_INSTANCE.delete_lock(name, uid)
        if res:
            self.send_status(204)
        else:
            self.send_error(status_code=404, message="lock not found")
            return
